clear;
%tic;
%number of iterations
K           =   10;

%number of Monte Carlo paths
Npath       =   10000;

%terminal time
T           =   30;
dt          =   1/12;

%number of temporal mesh points
M           =   round(T/dt);

%cir coefficients
theta       =   0.06;
kappa       =   0.25;
sigma       =   0.1;

alpha       =   0.5*sqrt(kappa*kappa+2*sigma*sigma);
C           =   @(u) sinh(alpha*u)/(alpha*cosh(alpha*u)+0.5*kappa*sinh(alpha*u));
A           =   @(u) -2*kappa*theta/(sigma*sigma)*log(alpha*exp(0.5*kappa*u)/(alpha*cosh(alpha*u)+0.5*kappa*sinh(alpha*u)));
fi          =   @(t,x) exp(-x*C(t)-A(t));

%%% quantiles of the CIR invariant distribution %%%
pd          =   makedist('Gamma','a',2*theta*kappa/(sigma*sigma),'b',sigma*sigma/(2*kappa));
pdfg        =   pdf(pd,0:0.001:0.2);
endpoint    =   roundn(icdf(pd,[0.025,0.975]),-3);

%r mesh to compute M
rmax        =   endpoint(2);
rmin        =   endpoint(1);
dr          =   0.01;
Nr          =   round((rmax-rmin)/dr);

%r mesh to compute M0
rmax0       =   1;
rmin0       =   0.005;
dr0         =   0.005;
Nr0         =   round((rmax0-rmin0)/dr0);

%remaining balance function
p           =   @(t,m) 1-exp(-m*(T-t));
pm          =   @(t,m) (T-t)*exp(-m*(T-t));

%prepayment intensity
lambda      =   0.045;
epsilon     =   5;
epsilon0    =   5;
ell         =   0.09;
%gamma_e     =   @(x,y,z) lambda+epsilon0*subplus(ell-x)+epsilon*subplus(y-z);
gamma_e     =   @(x,y,z) lambda+epsilon*subplus(y-z);
g1          =   @(y,z) subplus(y-z);
